@@ -31,6 +31,7 @@ typedef struct _AttachInfo	AttachInfo;
 #include "procmime.h"
 #include "addressbook.h"
 #include "prefs_account.h"
+#include "undo.h"
 
 typedef enum
 {
@@ -147,6 +148,8 @@ struct _Compose
 	PrefsAccount *account;
 	PrefsAccount *orig_account;
 
+	UndoMain *undostruct;
+
 	/* external editor */
 	gchar *exteditor_file;
 	pid_t  exteditor_pid;
@@ -166,7 +169,7 @@ struct _AttachInfo
 void compose_new		(PrefsAccount	*account);
 
 void compose_new_with_recipient	(PrefsAccount	*account,
-				 const gchar	*to);
+				 const gchar	*mailto);
 
 void compose_reply		(MsgInfo	*msginfo,
 				 gboolean	 quote,
