@@ -87,8 +87,10 @@
 #define BORDER_WIDTH			2
 #define CTREE_INDENT			18
 #define FOLDER_SPACING			4
+#define MAX_ENTRY_LENGTH		8191
 #define COLOR_DIM			35000
-#define UI_REFRESH_INTERVAL		50000
+#define UI_REFRESH_INTERVAL		50000	/* usec */
+#define SESSION_TIMEOUT			60	/* sec */
 #define MAX_HISTORY_SIZE		16
 
 #define DEFAULT_NORMAL_FONT	"-*-helvetica-medium-r-normal--12-*," \
