@@ -42,11 +42,13 @@
 #include "mainwindow.h"
 #include "folderview.h"
 #include "summaryview.h"
+#include "summary_search.h"
 #include "inputdialog.h"
 #include "grouplistdialog.h"
 #include "manage_window.h"
 #include "alertpanel.h"
 #include "menu.h"
+#include "stock_pixmap.h"
 #include "procmsg.h"
 #include "utils.h"
 #include "gtkutils.h"
@@ -56,12 +58,6 @@
 #include "folder.h"
 #include "inc.h"
 
-#include "pixmaps/inbox.xpm"
-#include "pixmaps/outbox.xpm"
-#include "pixmaps/dir-close.xpm"
-#include "pixmaps/dir-open.xpm"
-#include "pixmaps/trash.xpm"
-
 typedef enum
 {
 	COL_FOLDER	= 0,
@@ -188,6 +184,10 @@ static void folderview_rm_news_server_cb(FolderView	*folderview,
 					 guint		 action,
 					 GtkWidget	*widget);
 
+static void folderview_search_cb	(FolderView	*folderview,
+					 guint		 action,
+					 GtkWidget	*widget);
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
@@ -213,11 +213,13 @@ static GtkItemFactoryEntry folderview_mail_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, folderview_rename_folder_cb, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_delete_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
-	{N_("/Re_scan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/_Check for new messages"),
+					NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/R_escan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _mailbox"),	NULL, folderview_remove_mailbox_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
 };
 
@@ -227,23 +229,31 @@ static GtkItemFactoryEntry folderview_imap_popup_entries[] =
 	{N_("/_Rename folder..."),	NULL, NULL, 0, NULL},
 	{N_("/_Delete folder"),		NULL, folderview_rm_imap_folder_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Update folder tree"),	NULL, folderview_update_tree_cb, 0, NULL},
-	{N_("/Re_scan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
+	{N_("/_Check for new messages"),
+					NULL, folderview_update_tree_cb, 0, NULL},
+	{N_("/R_escan folder tree"),	NULL, folderview_update_tree_cb, 1, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/Remove _IMAP4 account"),	NULL, folderview_rm_imap_server_cb, 0, NULL},
 	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
 	{N_("/_Property..."),		NULL, NULL, 0, NULL}
 };
 
 static GtkItemFactoryEntry folderview_news_popup_entries[] =
 {
-	{N_("/_Subscribe to newsgroup..."),
-					 NULL, folderview_new_news_group_cb, 0, NULL},
-	{N_("/_Remove newsgroup"),	 NULL, folderview_rm_news_group_cb, 0, NULL},
-	{N_("/---"),			 NULL, NULL, 0, "<Separator>"},
-	{N_("/Remove _news account"),	 NULL, folderview_rm_news_server_cb, 0, NULL},
-	{N_("/---"),			 NULL, NULL, 0, "<Separator>"},
-	{N_("/_Property..."),		 NULL, NULL, 0, NULL}
+	{N_("/Su_bscribe to newsgroup..."),
+					NULL, folderview_new_news_group_cb, 0, NULL},
+	{N_("/_Remove newsgroup"),	NULL, folderview_rm_news_group_cb, 0, NULL},
+#if 0
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Check for new messages"),
+					NULL, folderview_update_tree_cb, 0, NULL},
+#endif
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/Remove _news account"),	NULL, folderview_rm_news_server_cb, 0, NULL},
+	{N_("/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Search messages..."),	NULL, folderview_search_cb, 0, NULL},
+	{N_("/_Property..."),		NULL, NULL, 0, NULL}
 };
 
 
@@ -389,11 +399,14 @@ void folderview_init(FolderView *folderview)
 {
 	GtkWidget *ctree = folderview->ctree;
 
-	PIXMAP_CREATE(ctree, inboxxpm, inboxxpmmask, inbox_xpm);
-	PIXMAP_CREATE(ctree, outboxxpm, outboxxpmmask, outbox_xpm);
-	PIXMAP_CREATE(ctree, folderxpm, folderxpmmask, dir_close_xpm);
-	PIXMAP_CREATE(ctree, folderopenxpm, folderopenxpmmask, dir_open_xpm);
-	PIXMAP_CREATE(ctree, trashxpm, trashxpmmask, trash_xpm);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_INBOX, &inboxxpm, &inboxxpmmask);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_OUTBOX,
+			 &outboxxpm, &outboxxpmmask);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_DIR_CLOSE,
+			 &folderxpm, &folderxpmmask);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_DIR_OPEN,
+			 &folderopenxpm, &folderopenxpmmask);
+	stock_pixmap_gdk(ctree, STOCK_PIXMAP_TRASH, &trashxpm, &trashxpmmask);
 
 	if (!normalfont)
 		normalfont = gdk_fontset_load(prefs_common.normalfont);
@@ -1132,6 +1145,7 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 	gboolean update_tree   = FALSE;
 	gboolean rescan_tree   = FALSE;
 	gboolean remove_tree   = FALSE;
+	gboolean search_folder = FALSE;
 
 	if (!event) return;
 
@@ -1177,6 +1191,8 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		new_folder = TRUE;
 		if (item->parent == NULL)
 			update_tree = remove_tree = TRUE;
+		else
+			search_folder = TRUE;
 		if (FOLDER_IS_LOCAL(folder) || FOLDER_TYPE(folder) == F_IMAP) {
 			if (item->parent == NULL)
 				update_tree = rescan_tree = TRUE;
@@ -1197,24 +1213,30 @@ static void folderview_button_pressed(GtkWidget *ctree, GdkEventButton *event,
 		SET_SENS(mail_factory, "/Create new folder...", new_folder);
 		SET_SENS(mail_factory, "/Rename folder...", rename_folder);
 		SET_SENS(mail_factory, "/Delete folder", delete_folder);
-		SET_SENS(mail_factory, "/Update folder tree", update_tree);
+		SET_SENS(mail_factory, "/Check for new messages", update_tree);
 		SET_SENS(mail_factory, "/Rescan folder tree", rescan_tree);
 		SET_SENS(mail_factory, "/Remove mailbox", remove_tree);
+		SET_SENS(mail_factory, "/Search messages...", search_folder);
 	} else if (FOLDER_TYPE(folder) == F_IMAP) {
 		popup = folderview->imap_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
 		SET_SENS(imap_factory, "/Create new folder...", new_folder);
 		SET_SENS(imap_factory, "/Rename folder...", rename_folder);
 		SET_SENS(imap_factory, "/Delete folder", delete_folder);
-		SET_SENS(imap_factory, "/Update folder tree", update_tree);
+		SET_SENS(imap_factory, "/Check for new messages", update_tree);
 		SET_SENS(imap_factory, "/Rescan folder tree", rescan_tree);
 		SET_SENS(imap_factory, "/Remove IMAP4 account", remove_tree);
+		SET_SENS(imap_factory, "/Search messages...", search_folder);
 	} else if (FOLDER_TYPE(folder) == F_NEWS) {
 		popup = folderview->news_popup;
 		menu_set_insensitive_all(GTK_MENU_SHELL(popup));
 		SET_SENS(news_factory, "/Subscribe to newsgroup...", new_folder);
 		SET_SENS(news_factory, "/Remove newsgroup", delete_folder);
+#if 0
+		SET_SENS(news_factory, "/Check for new messages", update_tree);
+#endif
 		SET_SENS(news_factory, "/Remove news account", remove_tree);
+		SET_SENS(news_factory, "/Search messages...", search_folder);
 	} else
 		return;
 
@@ -1775,55 +1797,88 @@ static void folderview_new_news_group_cb(FolderView *folderview, guint action,
 	GtkCTree *ctree = GTK_CTREE(folderview->ctree);
 	gchar *text[N_FOLDER_COLS] = {NULL, "0", "0", "0"};
 	GtkCTreeNode *servernode, *node;
+	Folder *folder;
 	FolderItem *item;
+	FolderItem *rootitem;
 	FolderItem *newitem;
-	gchar *new_group;
-	const gchar *server;
+	GSList *new_subscr;
+	GSList *cur;
+	GNode *gnode;
 
 	if (!folderview->selected) return;
 
 	item = gtk_ctree_node_get_row_data(ctree, folderview->selected);
 	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->folder->type == F_NEWS);
-	g_return_if_fail(item->folder->account != NULL);
-
-	new_group = grouplist_dialog(item->folder);
-	if (!new_group) return;
+	folder = item->folder;
+	g_return_if_fail(folder != NULL);
+	g_return_if_fail(folder->type == F_NEWS);
+	g_return_if_fail(folder->account != NULL);
 
 	if (GTK_CTREE_ROW(folderview->selected)->parent != NULL)
 		servernode = GTK_CTREE_ROW(folderview->selected)->parent;
 	else
 		servernode = folderview->selected;
 
-	if (folderview_find_by_name(ctree, servernode, new_group)) {
-		alertpanel_error(_("The newsgroup `%s' already exists."),
-				 new_group);
-		g_free(new_group);
-		return;
+	rootitem = gtk_ctree_node_get_row_data(ctree, servernode);
+
+	new_subscr = grouplist_dialog(folder);
+
+	/* remove unsubscribed newsgroups */
+	for (gnode = folder->node->children; gnode != NULL; ) {
+		GNode *next = gnode->next;
+
+		item = FOLDER_ITEM(gnode->data);
+		if (g_slist_find_custom(new_subscr, item->name,
+					(GCompareFunc)g_strcasecmp) != NULL) {
+			gnode = next;
+			continue;
+		}
+
+		node = gtk_ctree_find_by_row_data(ctree, servernode, item);
+		if (!node) {
+			gnode = next;
+			continue;
+		}
+
+		if (folderview->opened == node) {
+			summary_clear_all(folderview->summaryview);
+			folderview->opened = NULL;
+		}
+
+		folder_item_remove(item);
+		gtk_ctree_remove_node(ctree, node);
+
+		gnode = next;
 	}
 
 	gtk_clist_freeze(GTK_CLIST(ctree));
 
-	text[COL_FOLDER] = new_group;
-	node = gtk_ctree_insert_node(ctree, servernode, NULL, text,
-				     FOLDER_SPACING,
-				     folderxpm, folderxpmmask,
-				     folderopenxpm, folderopenxpmmask,
-				     FALSE, FALSE);
-	gtk_ctree_expand(ctree, servernode);
+	/* add subscribed newsgroups */
+	for (cur = new_subscr; cur != NULL; cur = cur->next) {
+		gchar *name = (gchar *)cur->data;
 
-	item = gtk_ctree_node_get_row_data(ctree, servernode);
-	server = item->folder->account->nntp_server;
+		if (folderview_find_by_name(ctree, servernode, name) != NULL)
+			continue;
+
+		text[COL_FOLDER] = name;
+		node = gtk_ctree_insert_node(ctree, servernode, NULL, text,
+					     FOLDER_SPACING,
+					     folderxpm, folderxpmmask,
+					     folderopenxpm, folderopenxpmmask,
+					     FALSE, FALSE);
+		gtk_ctree_expand(ctree, servernode);
+
+		newitem = folder_item_new(name, name);
+		folder_item_append(rootitem, newitem);
+		gtk_ctree_node_set_row_data(ctree, node, newitem);
+	}
 
-	newitem = folder_item_new(new_group, new_group);
-	g_free(new_group);
-	folder_item_append(item, newitem);
-	gtk_ctree_node_set_row_data(ctree, node, newitem);
 	gtk_ctree_sort_node(ctree, servernode);
-
 	gtk_clist_thaw(GTK_CLIST(ctree));
 
+	slist_free_strings(new_subscr);
+	g_slist_free(new_subscr);
+
 	folder_write_list();
 }
 
@@ -1900,6 +1955,12 @@ static void folderview_rm_news_server_cb(FolderView *folderview, guint action,
 	folder_write_list();
 }
 
+static void folderview_search_cb(FolderView *folderview, guint action,
+				 GtkWidget *widget)
+{
+	summary_search(folderview->summaryview);
+}
+
 static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 					  GdkDragContext *context,
 					  gint            x,
@@ -1936,7 +1997,13 @@ static gboolean folderview_drag_motion_cb(GtkWidget      *widget,
 	}
 
 	if (acceptable) {
+		gtk_signal_handler_block_by_func
+			(GTK_OBJECT(widget),
+			 GTK_SIGNAL_FUNC(folderview_selected), folderview);
 		gtk_ctree_select(GTK_CTREE(widget), node);
+		gtk_signal_handler_unblock_by_func
+			(GTK_OBJECT(widget),
+			 GTK_SIGNAL_FUNC(folderview_selected), folderview);
 		gdk_drag_status(context, context->suggested_action, time);
 	} else {
 		gtk_ctree_select(GTK_CTREE(widget), folderview->opened);
