@@ -21,6 +21,8 @@
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <glib.h>
 #include <stdio.h>
 #include <string.h>
@@ -28,6 +30,7 @@
 #include <dirent.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <time.h>
 
 #include "intl.h"
 #include "imap.h"
@@ -48,6 +51,21 @@
 #define IMAPS_PORT	993
 #endif
 
+#define QUOTE_IF_REQUIRED(out, str) \
+{ \
+	if (*str != '"' && strchr(str, ' ')) { \
+		gchar *__tmp; \
+		gint len; \
+ \
+		len = strlen(str) + 3; \
+		Xalloca(__tmp, len, return IMAP_ERROR); \
+		g_snprintf(__tmp, len, "\"%s\"", str); \
+		out = __tmp; \
+	} else { \
+		Xstrdup_a(out, str, return IMAP_ERROR); \
+	} \
+}
+
 static GList *session_list = NULL;
 
 static gint imap_cmd_count = 0;
@@ -56,7 +74,7 @@ static IMAPSession *imap_session_get	(Folder		*folder);
 static gchar *imap_query_password	(const gchar	*server,
 					 const gchar	*user);
 
-static void imap_scan_tree_recursive	(IMAPSession	*session,
+static gint imap_scan_tree_recursive	(IMAPSession	*session,
 					 FolderItem	*item,
 					 IMAPNameSpace	*namespace);
 static GSList *imap_parse_list		(IMAPSession	*session,
@@ -218,6 +236,8 @@ static gchar *get_quoted			(const gchar	*src,
 						 gint		 len);
 static gchar *search_array_contain_str		(GPtrArray	*array,
 						 gchar		*str);
+static gchar *search_array_str			(GPtrArray	*array,
+						 gchar		*str);
 static void imap_path_separator_subst		(gchar		*str,
 						 gchar		 separator);
 
@@ -250,9 +270,17 @@ static IMAPSession *imap_session_get(Folder *folder)
 					 folder->account->userid,
 					 folder->account->passwd);
 #endif
-		if (rfolder->session)
+		if (rfolder->session) {
 			imap_parse_namespace(IMAP_SESSION(rfolder->session),
 					     IMAP_FOLDER(folder));
+			rfolder->session->last_access_time = time(NULL);
+		}
+		statusbar_pop_all();
+		return IMAP_SESSION(rfolder->session);
+	}
+
+	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT) {
+		rfolder->session->last_access_time = time(NULL);
 		statusbar_pop_all();
 		return IMAP_SESSION(rfolder->session);
 	}
@@ -278,6 +306,8 @@ static IMAPSession *imap_session_get(Folder *folder)
 					     IMAP_FOLDER(folder));
 	}
 
+	if (rfolder->session)
+		rfolder->session->last_access_time = time(NULL);
 	statusbar_pop_all();
 	return IMAP_SESSION(rfolder->session);
 }
@@ -336,12 +366,13 @@ Session *imap_session_new(const gchar *server, gushort port,
 	}
 
 	session = g_new(IMAPSession, 1);
-	SESSION(session)->type      = SESSION_IMAP;
-	SESSION(session)->server    = g_strdup(server);
-	SESSION(session)->sock      = imap_sock;
-	SESSION(session)->connected = TRUE;
-	SESSION(session)->phase     = SESSION_READY;
-	SESSION(session)->data      = NULL;
+	SESSION(session)->type             = SESSION_IMAP;
+	SESSION(session)->server           = g_strdup(server);
+	SESSION(session)->sock             = imap_sock;
+	SESSION(session)->connected        = TRUE;
+	SESSION(session)->phase            = SESSION_READY;
+	SESSION(session)->last_access_time = time(NULL);
+	SESSION(session)->data             = NULL;
 	session->mbox = NULL;
 
 	session_list = g_list_append(session_list, session);
@@ -807,42 +838,53 @@ void imap_scan_tree(Folder *folder)
 		imap_create_trash(folder);
 }
 
-static void imap_scan_tree_recursive(IMAPSession *session,
-				     FolderItem *item,
+static gint imap_scan_tree_recursive(IMAPSession *session, FolderItem *item,
 				     IMAPNameSpace *namespace)
 {
 	IMAPFolder *imapfolder;
 	FolderItem *new_item;
 	GSList *item_list, *cur;
 	gchar *real_path;
+	gchar *wildcard_path;
+	gchar separator = '/';
+	gchar wildcard[3];
 
-	g_return_if_fail(item != NULL);
-	g_return_if_fail(item->folder != NULL);
-	g_return_if_fail(item->no_sub == FALSE);
+	g_return_val_if_fail(item != NULL, -1);
+	g_return_val_if_fail(item->folder != NULL, -1);
+	g_return_val_if_fail(item->no_sub == FALSE, -1);
 
 	imapfolder = IMAP_FOLDER(item->folder);
 
+	if (namespace && namespace->separator)
+		separator = namespace->separator;
+
 	if (item->folder->ui_func)
 		item->folder->ui_func(item->folder, item,
 				      item->folder->ui_func_data);
 
 	if (item->path) {
+		wildcard[0] = separator;
+		wildcard[1] = '%';
+		wildcard[2] = '\0';
 		real_path = imap_get_real_path(imapfolder, item->path);
-		imap_cmd_gen_send(SESSION(session)->sock, "LIST \"\" %s%c%%",
-				  real_path,
-				  namespace && namespace->separator
-				  ? namespace->separator : '/');
 	} else {
+		wildcard[0] = '%';
+		wildcard[1] = '\0';
 		real_path = g_strdup(namespace && namespace->name
 				     ? namespace->name : "");
-		imap_cmd_gen_send(SESSION(session)->sock, "LIST \"\" %s%%",
-				  real_path);
 	}
 
-	strtailchomp(real_path, namespace && namespace->separator
-		     ? namespace->separator : '/');
+	Xstrcat_a(wildcard_path, real_path, wildcard,
+		  {g_free(real_path); return IMAP_ERROR;});
+	QUOTE_IF_REQUIRED(wildcard_path, wildcard_path);
 
+	imap_cmd_gen_send(SESSION(session)->sock, "LIST \"\" %s",
+			  wildcard_path);
+
+	strtailchomp(real_path, separator);
 	item_list = imap_parse_list(session, real_path);
+	g_free(real_path);
+
 	for (cur = item_list; cur != NULL; cur = cur->next) {
 		new_item = cur->data;
 		if (!strcmp(new_item->path, "INBOX")) {
@@ -865,6 +907,8 @@ static void imap_scan_tree_recursive(IMAPSession *session,
 		if (new_item->no_sub == FALSE)
 			imap_scan_tree_recursive(session, new_item, namespace);
 	}
+
+	return IMAP_SUCCESS;
 }
 
 static GSList *imap_parse_list(IMAPSession *session, const gchar *path)
@@ -1568,6 +1612,8 @@ static MsgFlags imap_parse_flags(const gchar *flag_str)
 			MSG_SET_PERM_FLAGS(flags, MSG_DELETED);
 		} else if (g_strncasecmp(p, "Flagged", 7) == 0) {
 			MSG_SET_PERM_FLAGS(flags, MSG_MARKED);
+		} else if (g_strncasecmp(p, "Answered", 8) == 0) {
+			MSG_SET_PERM_FLAGS(flags, MSG_REPLIED);
 		}
 	}
 
@@ -1651,6 +1697,7 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 			cur_pos = imap_parse_address(sock, cur_pos + 1,
 						     &tmp_from, &tmp_fromname,
 						     line_str);
+			g_return_val_if_fail(cur_pos != NULL, NULL);
 			Xstrdup_a(from, tmp_from,
 				  {g_free(tmp_from); g_free(tmp_fromname);
 				   return NULL;});
@@ -1665,6 +1712,7 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 	g_return_val_if_fail(*cur_pos == ' ', NULL); \
 	cur_pos = imap_parse_address(sock, cur_pos + 1, NULL, NULL, \
 				     line_str); \
+	g_return_val_if_fail(cur_pos != NULL, NULL); \
 }
 
 			/* skip sender and reply-to */
@@ -1674,6 +1722,7 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 			g_return_val_if_fail(*cur_pos == ' ', NULL);
 			cur_pos = imap_parse_address(sock, cur_pos + 1,
 						     &tmp_to, NULL, line_str);
+			g_return_val_if_fail(cur_pos != NULL, NULL);
 			Xstrdup_a(to, tmp_to, {g_free(tmp_to); return NULL;});
 			g_free(tmp_to);
 
@@ -1686,6 +1735,7 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 			g_return_val_if_fail(*cur_pos == ' ', NULL);
 			cur_pos = imap_parse_atom
 				(sock, cur_pos, buf, sizeof(buf), line_str);
+			g_return_val_if_fail(cur_pos != NULL, NULL);
 			if (buf[0] != '\0') {
 				eliminate_parenthesis(buf, '(', ')');
 				extract_parenthesis(buf, '<', '>');
@@ -1696,6 +1746,7 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 			g_return_val_if_fail(*cur_pos == ' ', NULL);
 			cur_pos = imap_parse_atom
 				(sock, cur_pos, buf, sizeof(buf), line_str);
+			g_return_val_if_fail(cur_pos != NULL, NULL);
 			if (buf[0] != '\0') {
 				extract_parenthesis(buf, '<', '>');
 				remove_space(buf);
@@ -1726,6 +1777,72 @@ static MsgInfo *imap_parse_envelope(SockInfo *sock, GString *line_str)
 	return msginfo;
 }
 
+gint imap_msg_set_perm_flags(MsgInfo *msginfo, MsgPermFlags flags)
+{
+	Folder *folder;
+	IMAPSession *session;
+	IMAPFlags iflags = 0;
+	gint ok = IMAP_SUCCESS;
+
+	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(MSG_IS_IMAP(msginfo->flags), -1);
+	g_return_val_if_fail(msginfo->folder != NULL, -1);
+	g_return_val_if_fail(msginfo->folder->folder != NULL, -1);
+
+	folder = msginfo->folder->folder;
+	g_return_val_if_fail(folder->type == F_IMAP, -1);
+
+	session = imap_session_get(folder);
+	if (!session) return -1;
+
+	if (flags & MSG_MARKED)  iflags |= IMAP_FLAG_FLAGGED;
+	if (flags & MSG_REPLIED) iflags |= IMAP_FLAG_ANSWERED;
+	if (iflags) {
+		ok = imap_set_message_flags(session, msginfo->msgnum,
+					    msginfo->msgnum, iflags, TRUE);
+		if (ok != IMAP_SUCCESS) return ok;
+	}
+
+	if (flags & MSG_UNREAD)
+		ok = imap_set_message_flags(session, msginfo->msgnum,
+					    msginfo->msgnum, IMAP_FLAG_SEEN,
+					    FALSE);
+	return ok;
+}
+
+gint imap_msg_unset_perm_flags(MsgInfo *msginfo, MsgPermFlags flags)
+{
+	Folder *folder;
+	IMAPSession *session;
+	IMAPFlags iflags = 0;
+	gint ok = IMAP_SUCCESS;
+
+	g_return_val_if_fail(msginfo != NULL, -1);
+	g_return_val_if_fail(MSG_IS_IMAP(msginfo->flags), -1);
+	g_return_val_if_fail(msginfo->folder != NULL, -1);
+	g_return_val_if_fail(msginfo->folder->folder != NULL, -1);
+
+	folder = msginfo->folder->folder;
+	g_return_val_if_fail(folder->type == F_IMAP, -1);
+
+	session = imap_session_get(folder);
+	if (!session) return -1;
+
+	if (flags & MSG_MARKED)  iflags |= IMAP_FLAG_FLAGGED;
+	if (flags & MSG_REPLIED) iflags |= IMAP_FLAG_ANSWERED;
+	if (iflags) {
+		ok = imap_set_message_flags(session, msginfo->msgnum,
+					    msginfo->msgnum, iflags, FALSE);
+		if (ok != IMAP_SUCCESS) return ok;
+	}
+
+	if (flags & MSG_UNREAD)
+		ok = imap_set_message_flags(session, msginfo->msgnum,
+					    msginfo->msgnum, IMAP_FLAG_SEEN,
+					    TRUE);
+	return ok;
+}
+
 static gint imap_set_message_flags(IMAPSession *session,
 				   guint32 first_uid,
 				   guint32 last_uid,
@@ -1805,18 +1922,6 @@ catch:
 	return ok;
 }
 
-#define QUOTE_IF_REQUIRED(out, str) \
-{ \
-	if (*str != '"' && strchr(str, ' ')) { \
-		gint len; \
-		len = strlen(str) + 3; \
-		Xalloca(out, len, return IMAP_ERROR); \
-		g_snprintf(out, len, "\"%s\"", str); \
-	} else { \
-		Xstrdup_a(out, str, return IMAP_ERROR); \
-	} \
-}
-
 static gint imap_status(IMAPSession *session, IMAPFolder *folder,
 			const gchar *path,
 			gint *messages, gint *recent, gint *unseen,
@@ -1840,7 +1945,7 @@ static gint imap_status(IMAPSession *session, IMAPFolder *folder,
 	ok = imap_cmd_ok(SESSION(session)->sock, argbuf);
 	if (ok != IMAP_SUCCESS) THROW(ok);
 
-	str = search_array_contain_str(argbuf, "STATUS");
+	str = search_array_str(argbuf, "STATUS");
 	if (!str) THROW(IMAP_ERROR);
 
 	str = strchr(str, '(');
@@ -1922,7 +2027,7 @@ static gint imap_cmd_namespace(SockInfo *sock, gchar **ns_str)
 	imap_cmd_gen_send(sock, "NAMESPACE");
 	if ((ok = imap_cmd_ok(sock, argbuf)) != IMAP_SUCCESS) THROW(ok);
 
-	str = search_array_contain_str(argbuf, "NAMESPACE");
+	str = search_array_str(argbuf, "NAMESPACE");
 	if (!str) THROW(IMAP_ERROR);
 
 	*ns_str = g_strdup(str);
@@ -2101,14 +2206,16 @@ static gint imap_cmd_append(SockInfo *sock, const gchar *destfolder,
 {
 	gint ok;
 	gint size;
+	gchar *destfolder_;
 
 	g_return_val_if_fail(file != NULL, IMAP_ERROR);
 
 	size = get_file_size(file);
-	imap_cmd_gen_send(sock, "APPEND %s {%d}", destfolder, size);
+	QUOTE_IF_REQUIRED(destfolder_, destfolder);
+	imap_cmd_gen_send(sock, "APPEND %s {%d}", destfolder_, size);
 	ok = imap_cmd_ok(sock, NULL);
 	if (ok != IMAP_SUCCESS) {
-		log_warning(_("can't append %s to %s\n"), file, destfolder);
+		log_warning(_("can't append %s to %s\n"), file, destfolder_);
 		return -1;
 	}
 
@@ -2294,6 +2401,24 @@ static gchar *search_array_contain_str(GPtrArray *array, gchar *str)
 	return NULL;
 }
 
+static gchar *search_array_str(GPtrArray *array, gchar *str)
+{
+	gint i;
+	gint len;
+
+	len = strlen(str);
+
+	for (i = 0; i < array->len; i++) {
+		gchar *tmp;
+
+		tmp = g_ptr_array_index(array, i);
+		if (!strncmp(tmp, str, len))
+			return tmp;
+	}
+
+	return NULL;
+}
+
 static void imap_path_separator_subst(gchar *str, gchar separator)
 {
 	if (separator && separator != '/')
