@@ -25,9 +25,11 @@
 #endif
 
 #include <glib.h>
+#include <time.h>
 
 #include "folder.h"
 #include "session.h"
+#include "procmsg.h"
 
 typedef struct _IMAPSession	IMAPSession;
 typedef struct _IMAPNameSpace	IMAPNameSpace;
@@ -41,6 +43,7 @@ struct _IMAPSession
 	Session session;
 
 	gchar *mbox;
+	time_t last_access_time;
 };
 
 struct _IMAPNameSpace
@@ -131,4 +134,9 @@ FolderItem *imap_create_folder		(Folder		*folder,
 gint imap_remove_folder			(Folder		*folder,
 					 FolderItem	*item);
 
+gint imap_msg_set_perm_flags		(MsgInfo	*msginfo,
+					 MsgPermFlags	 flags);
+gint imap_msg_unset_perm_flags		(MsgInfo	*msginfo,
+					 MsgPermFlags	 flags);
+
 #endif /* __IMAP_H__ */
