@@ -51,6 +51,7 @@
 #include "message_search.h"
 #include "headerview.h"
 #include "menu.h"
+#include "stock_pixmap.h"
 #include "folder.h"
 #include "inc.h"
 #include "compose.h"
@@ -381,15 +382,16 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_File/_Save as..."),		NULL, save_as_cb, 0, NULL},
 	{N_("/_File/_Print..."),		"<alt>P", print_cb, 0, NULL},
 	{N_("/_File/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_File/_Close"),			"<alt>W", app_exit_cb, 0, NULL},
+	/* {N_("/_File/_Close"),		"<alt>W", app_exit_cb, 0, NULL}, */
 	{N_("/_File/E_xit"),			"<alt>Q", app_exit_cb, 0, NULL},
 
 	{N_("/_Edit"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Edit/_Copy"),			"<control>C", copy_cb, 0, NULL},
 	{N_("/_Edit/Select _all"),		"<control>A", allsel_cb, 0, NULL},
 	{N_("/_Edit/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Edit/_Find in current message"),	"<control>F", search_cb, 0, NULL},
-	{N_("/_Edit/_Search folder"),		"<control>S", search_cb, 1, NULL},
+	{N_("/_Edit/_Find in current message..."),
+						"<control>F", search_cb, 0, NULL},
+	{N_("/_Edit/_Search messages..."),	"<control>S", search_cb, 1, NULL},
 
 	{N_("/_View"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_View/_Folder tree"),		NULL, toggle_folder_cb, 0, "<ToggleItem>"},
@@ -399,17 +401,60 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_View/_Toolbar/_Icon"),		NULL, toggle_toolbar_cb, TOOLBAR_ICON, "/View/Toolbar/Icon and text"},
 	{N_("/_View/_Toolbar/_Text"),		NULL, toggle_toolbar_cb, TOOLBAR_TEXT, "/View/Toolbar/Icon and text"},
 	{N_("/_View/_Toolbar/_None"),		NULL, toggle_toolbar_cb, TOOLBAR_NONE, "/View/Toolbar/Icon and text"},
-	{N_("/_View/_Status bar"),		NULL, toggle_statusbar_cb, 0, "<ToggleItem>"},
+	{N_("/_View/Status _bar"),		NULL, toggle_statusbar_cb, 0, "<ToggleItem>"},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Separate f_older tree"),	NULL, separate_widget_cb, SEPARATE_FOLDER, "<ToggleItem>"},
 	{N_("/_View/Separate m_essage view"),	NULL, separate_widget_cb, SEPARATE_MESSAGE, "<ToggleItem>"},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_View/View _source"),		"<control>U", view_source_cb, 0, NULL},
-	{N_("/_View/Show all _header"),		"<control>H", header_window_show_cb, 0, NULL},
+	{N_("/_View/_Sort"),			NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/_Sort/Sort by _number"),	NULL, sort_summary_cb, SORT_BY_NUMBER, NULL},
+	{N_("/_View/_Sort/Sort by s_ize"),	NULL, sort_summary_cb, SORT_BY_SIZE, NULL},
+	{N_("/_View/_Sort/Sort by _date"),	NULL, sort_summary_cb, SORT_BY_DATE, NULL},
+	{N_("/_View/_Sort/Sort by _from"),	NULL, sort_summary_cb, SORT_BY_FROM, NULL},
+	{N_("/_View/_Sort/Sort by _subject"),NULL, sort_summary_cb, SORT_BY_SUBJECT, NULL},
+	{N_("/_View/_Sort/Sort by _color label"),
+						NULL, sort_summary_cb, SORT_BY_LABEL, NULL},
+	{N_("/_View/_Sort/Sort by _mark"),	NULL, sort_summary_cb, SORT_BY_MARK, NULL},
+	{N_("/_View/_Sort/Sort by _unread"),	NULL, sort_summary_cb, SORT_BY_UNREAD, NULL},
+	{N_("/_View/_Sort/Sort by a_ttachment"),
+						NULL, sort_summary_cb, SORT_BY_MIME, NULL},
+	{N_("/_View/_Sort/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Sort/_Attract by subject"),
+						NULL, attract_by_subject_cb, 0, NULL},
+	{N_("/_View/Th_read view"),		"<control>T",	     thread_cb, 0, NULL},
+	{N_("/_View/U_nthread view"),		"<shift><control>T", thread_cb, 1, NULL},
+	{N_("/_View/Set display _item..."),	NULL, set_display_item_cb, 0, NULL},
 	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/_Code set"),		NULL, NULL, 0, "<Branch>"},
 	{N_("/_View/_Code set/_Auto detect"),
 	 NULL, set_charset_cb, C_AUTO, "<RadioItem>"},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Go to"),		NULL, NULL, 0, "<Branch>"},
+	{N_("/_View/_Go to/_Prev message"),	NULL, prev_cb, 0, NULL},
+	{N_("/_View/_Go to/_Next message"),	NULL, next_cb, 0, NULL},
+	{N_("/_View/_Go to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Go to/P_rev unread message"),
+						NULL, prev_unread_cb, 0, NULL},
+	{N_("/_View/_Go to/N_ext unread message"),
+						NULL, next_unread_cb, 0, NULL},
+	{N_("/_View/_Go to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Go to/Prev _marked message"),
+						NULL, prev_marked_cb, 0, NULL},
+	{N_("/_View/_Go to/Next m_arked message"),
+						NULL, next_marked_cb, 0, NULL},
+	{N_("/_View/_Go to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Go to/Prev _labeled message"),
+						NULL, prev_labeled_cb, 0, NULL},
+	{N_("/_View/_Go to/Next la_beled message"),
+						NULL, next_labeled_cb, 0, NULL},
+	{N_("/_View/_Go to/---"),		NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Go to/Other _folder..."),	"<alt>G", goto_folder_cb, 0, NULL},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/Open in new _window"),	"<shift><control>N", open_msg_cb, 0, NULL},
+	{N_("/_View/_View source"),		"<control>U", view_source_cb, 0, NULL},
+	{N_("/_View/Show all _header"),		"<control>H", header_window_show_cb, 0, NULL},
+	{N_("/_View/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_View/_Update"),			"<alt>U", update_summary_cb,  0, NULL},
 
 #define CODESET_SEPARATOR \
 	{N_("/_View/_Code set/---"),		NULL, NULL, 0, "<Separator>"}
@@ -476,6 +521,11 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	 CODESET_ACTION(C_EUC_KR)},
 	{N_("/_View/_Code set/Korean (ISO-2022-KR)"),
 	 CODESET_ACTION(C_ISO_2022_KR)},
+	CODESET_SEPARATOR,
+	{N_("/_View/_Code set/Thai (TIS-620)"),
+	 CODESET_ACTION(C_TIS_620)},
+	{N_("/_View/_Code set/Thai (Windows-874)"),
+	 CODESET_ACTION(C_WINDOWS_874)},
 #endif
 
 #undef CODESET_SEPARATOR
@@ -503,6 +553,9 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Copy..."),		NULL, copy_to_cb, 0, NULL},
 	{N_("/_Message/_Delete"),		"<alt>D", delete_cb,  0, NULL},
 	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Message/Delete du_plicated messages"),
+						NULL, delete_duplicated_cb,   0, NULL},
+	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Message/_Mark"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Message/_Mark/_Mark"),		NULL, mark_cb,   0, NULL},
 	{N_("/_Message/_Mark/_Unmark"),		NULL, unmark_cb, 0, NULL},
@@ -511,60 +564,16 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Message/_Mark/Mark as rea_d"),
 						NULL, mark_as_read_cb, 0, NULL},
 	{N_("/_Message/_Mark/Mark all _read"),	NULL, mark_all_read_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Add sender to address boo_k"),
-						NULL, add_address_cb, 0, NULL},
-	{N_("/_Message/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Message/Open in new _window"),	"<shift><control>N", open_msg_cb, 0, NULL},
-
-	{N_("/_Summary"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Summary/_Delete duplicated messages"),
-						NULL, delete_duplicated_cb,   0, NULL},
-	{N_("/_Summary/_Filter messages"),	NULL, filter_cb, 0, NULL},
-	{N_("/_Summary/E_xecute"),		"<alt>X", execute_summary_cb, 0, NULL},
-	{N_("/_Summary/_Update"),		"<alt>U", update_summary_cb,  0, NULL},
-	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/Go _to"),		NULL, NULL, 0, "<Branch>"},
-	{N_("/_Summary/Go _to/_Prev message"),	NULL, prev_cb, 0, NULL},
-	{N_("/_Summary/Go _to/_Next message"),	NULL, next_cb, 0, NULL},
-	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/Go _to/P_rev unread message"),
-						NULL, prev_unread_cb, 0, NULL},
-	{N_("/_Summary/Go _to/N_ext unread message"),
-						NULL, next_unread_cb, 0, NULL},
-	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/Go _to/Prev _marked message"),
-						NULL, prev_marked_cb, 0, NULL},
-	{N_("/_Summary/Go _to/Next m_arked message"),
-						NULL, next_marked_cb, 0, NULL},
-	{N_("/_Summary/Go _to/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/Go _to/Prev _labeled message"),
-						NULL, prev_labeled_cb, 0, NULL},
-	{N_("/_Summary/Go _to/Next la_beled message"),
-						NULL, next_labeled_cb, 0, NULL},
-	{N_("/_Summary/_Go to other folder..."),"<alt>G", goto_folder_cb, 0, NULL},
-	{N_("/_Summary/---"),			NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/_Sort"),			NULL, NULL, 0, "<Branch>"},
-	{N_("/_Summary/_Sort/Sort by _number"),	NULL, sort_summary_cb, SORT_BY_NUMBER, NULL},
-	{N_("/_Summary/_Sort/Sort by s_ize"),	NULL, sort_summary_cb, SORT_BY_SIZE, NULL},
-	{N_("/_Summary/_Sort/Sort by _date"),	NULL, sort_summary_cb, SORT_BY_DATE, NULL},
-	{N_("/_Summary/_Sort/Sort by _from"),	NULL, sort_summary_cb, SORT_BY_FROM, NULL},
-	{N_("/_Summary/_Sort/Sort by _subject"),NULL, sort_summary_cb, SORT_BY_SUBJECT, NULL},
-	{N_("/_Summary/_Sort/Sort by _color label"),
-						NULL, sort_summary_cb, SORT_BY_LABEL, NULL},
-	{N_("/_Summary/_Sort/Sort by _mark"),	NULL, sort_summary_cb, SORT_BY_MARK, NULL},
-	{N_("/_Summary/_Sort/Sort by _unread"),	NULL, sort_summary_cb, SORT_BY_UNREAD, NULL},
-	{N_("/_Summary/_Sort/Sort by a_ttachment"),
-						NULL, sort_summary_cb, SORT_BY_MIME, NULL},
-	{N_("/_Summary/_Sort/---"),		NULL, NULL, 0, "<Separator>"},
-	{N_("/_Summary/_Sort/_Attract by subject"),
-						NULL, attract_by_subject_cb, 0, NULL},
-	{N_("/_Summary/_Thread view"),		"<control>T",	     thread_cb, 0, NULL},
-	{N_("/_Summary/Unt_hread view"),	"<shift><control>T", thread_cb, 1, NULL},
-	{N_("/_Summary/Set display _item..."),	NULL, set_display_item_cb, 0, NULL},
 
 	{N_("/_Tool"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Tool/_Address book"),		"<alt>A", addressbook_open_cb, 0, NULL},
+	{N_("/_Tool/Add sender to address boo_k"),
+						NULL, add_address_cb, 0, NULL},
+	{N_("/_Tool/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Tool/_Filter messages"),		NULL, filter_cb, 0, NULL},
+	{N_("/_Tool/---"),			NULL, NULL, 0, "<Separator>"},
+	{N_("/_Tool/E_xecute"),			"<alt>X", execute_summary_cb, 0, NULL},
+	{N_("/_Tool/---"),			NULL, NULL, 0, "<Separator>"},
 	{N_("/_Tool/_Log window"),		"<alt>L", log_window_show_cb, 0, NULL},
 
 	{N_("/_Configuration"),			NULL, NULL, 0, "<Branch>"},
@@ -583,7 +592,7 @@ static GtkItemFactoryEntry mainwin_entries[] =
 	{N_("/_Configuration/C_hange current account"),
 						NULL, NULL, 0, "<Branch>"},
 
-	{N_("/_Help"),				NULL, NULL, 0, "<LastBranch>"},
+	{N_("/_Help"),				NULL, NULL, 0, "<Branch>"},
 	{N_("/_Help/_Manual"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_Help/_Manual/_English"),		NULL, manual_open_cb, MANUAL_LANG_EN, NULL},
 	{N_("/_Help/_Manual/_Japanese"),	NULL, manual_open_cb, MANUAL_LANG_JA, NULL},
@@ -775,9 +784,9 @@ MainWindow *main_window_create(SeparateType type)
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
 				       prefs_common.show_statusbar);
 
-	menu_set_sensitive(ifactory, "/Summary/Thread view",
+	menu_set_sensitive(ifactory, "/View/Thread view",
 			   prefs_common.enable_thread ? FALSE : TRUE);
-	menu_set_sensitive(ifactory, "/Summary/Unthread view",
+	menu_set_sensitive(ifactory, "/View/Unthread view",
 			   prefs_common.enable_thread ? TRUE : FALSE);
 
 	/* set account selection menu */
@@ -964,11 +973,11 @@ void main_window_separation_change(MainWindow *mainwin, SeparateType type)
 	gtk_widget_ref(folder_wid);
 	gtk_widget_ref(summary_wid);
 	gtk_widget_ref(message_wid);
-	gtk_container_remove
+	gtkut_container_remove
 		(GTK_CONTAINER(folder_wid->parent), folder_wid);
-	gtk_container_remove
+	gtkut_container_remove
 		(GTK_CONTAINER(summary_wid->parent), summary_wid);
-	gtk_container_remove
+	gtkut_container_remove
 		(GTK_CONTAINER(message_wid->parent), message_wid);
 
 	/* clean containers */
@@ -1056,7 +1065,7 @@ void main_window_empty_trash(MainWindow *mainwin, gboolean confirm)
 		Folder *folder;
 
 		folder = list->data;
-		if (folder->trash) {
+		if (folder->trash && folder->trash->total != 0) {
 			folder_item_scan(folder->trash);
 			folderview_update_item(folder->trash, TRUE);
 		}
@@ -1204,11 +1213,23 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/File/Empty trash"           , M_UNLOCKED},
 		{"/File/Save as...", M_SINGLE_TARGET_EXIST|M_UNLOCKED},
 		{"/File/Print..."  , M_TARGET_EXIST|M_UNLOCKED},
-		{"/File/Close", M_UNLOCKED},
+		/* {"/File/Close", M_UNLOCKED}, */
 		{"/File/Exit" , M_UNLOCKED},
 
-		{"/View/Show all header"      , M_SINGLE_TARGET_EXIST},
-		{"/View/View source"          , M_SINGLE_TARGET_EXIST},
+		{"/View/Sort"                      , M_MSG_EXIST},
+		{"/View/Thread view"               , M_UNTHREADED},
+		{"/View/Unthread view"             , M_THREADED},
+		{"/View/Go to"                     , M_MSG_EXIST},
+		{"/View/Go to/Prev message"        , M_MSG_EXIST},
+		{"/View/Go to/Next message"        , M_MSG_EXIST},
+		{"/View/Go to/Next unread message" , M_MSG_EXIST},
+		{"/View/Go to/Prev marked message" , M_MSG_EXIST},
+		{"/View/Go to/Next marked message" , M_MSG_EXIST},
+		{"/View/Go to/Prev labeled message", M_MSG_EXIST},
+		{"/View/Go to/Next labeled message", M_MSG_EXIST},
+		{"/View/Open in new window"        , M_SINGLE_TARGET_EXIST},
+		{"/View/Show all header"           , M_SINGLE_TARGET_EXIST},
+		{"/View/View source"               , M_SINGLE_TARGET_EXIST},
 
 		{"/Message/Get new mail"         , M_HAVE_ACCOUNT|M_UNLOCKED},
 		{"/Message/Get from all accounts", M_HAVE_ACCOUNT|M_UNLOCKED},
@@ -1218,28 +1239,16 @@ void main_window_set_menu_sensitive(MainWindow *mainwin)
 		{"/Message/Reply to all"         , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
 		{"/Message/Forward"              , M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
 		{"/Message/Forward as attachment", M_HAVE_ACCOUNT|M_SINGLE_TARGET_EXIST},
-		{"/Message/Open in new window"   , M_SINGLE_TARGET_EXIST},
 		{"/Message/Re-edit", M_HAVE_ACCOUNT|M_ALLOW_REEDIT},
 		{"/Message/Move...", M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
 		{"/Message/Copy...", M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
 		{"/Message/Delete" , M_TARGET_EXIST|M_EXEC|M_UNLOCKED},
 		{"/Message/Mark"   , M_TARGET_EXIST},
-		{"/Message/Add sender to address book", M_SINGLE_TARGET_EXIST},
+		{"/Message/Delete duplicated messages", M_MSG_EXIST|M_EXEC|M_UNLOCKED},
 
-		{"/Summary/Delete duplicated messages", M_MSG_EXIST|M_EXEC|M_UNLOCKED},
-		{"/Summary/Filter messages"           , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
-		{"/Summary/Execute"                   , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
-		{"/Summary/Go to"                     , M_MSG_EXIST},
-		{"/Summary/Go to/Prev message"        , M_MSG_EXIST},
-		{"/Summary/Go to/Next message"        , M_MSG_EXIST},
-		{"/Summary/Go to/Next unread message" , M_MSG_EXIST},
-		{"/Summary/Go to/Prev marked message" , M_MSG_EXIST},
-		{"/Summary/Go to/Next marked message" , M_MSG_EXIST},
-		{"/Summary/Go to/Prev labeled message", M_MSG_EXIST},
-		{"/Summary/Go to/Next labeled message", M_MSG_EXIST},
-		{"/Summary/Sort"                      , M_MSG_EXIST},
-		{"/Summary/Thread view"               ,	M_UNTHREADED},
-		{"/Summary/Unthread view"             , M_THREADED},
+		{"/Tool/Add sender to address book", M_SINGLE_TARGET_EXIST},
+		{"/Tool/Filter messages"           , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
+		{"/Tool/Execute"                   , M_MSG_EXIST|M_EXEC|M_UNLOCKED},
 
 		{"/Configuration", M_UNLOCKED},
 
@@ -1320,6 +1329,10 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	/* create separated window(s) if needed */
 	if (type & SEPARATE_FOLDER) {
 		folderwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_title(GTK_WINDOW(folderwin),
+				     _("Sylpheed - Folder View"));
+		gtk_window_set_wmclass(GTK_WINDOW(folderwin),
+				       "folder_view", "Sylpheed");
 		gtk_window_set_policy(GTK_WINDOW(folderwin),
 				      TRUE, TRUE, FALSE);
 		gtk_widget_set_usize(folderwin, -1,
@@ -1332,6 +1345,10 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	}
 	if (type & SEPARATE_MESSAGE) {
 		messagewin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_title(GTK_WINDOW(messagewin),
+				     _("Sylpheed - Message View"));
+		gtk_window_set_wmclass(GTK_WINDOW(messagewin),
+				       "message_view", "Sylpheed");
 		gtk_window_set_policy(GTK_WINDOW(messagewin),
 				      TRUE, TRUE, FALSE);
 		gtk_widget_set_usize
@@ -1465,33 +1482,10 @@ static void main_window_set_widgets(MainWindow *mainwin, SeparateType type)
 	debug_print(_("done.\n"));
 }
 
-#include "pixmaps/stock_mail_receive.xpm"
-#include "pixmaps/stock_mail_receive_all.xpm"
-#include "pixmaps/stock_mail_compose.xpm"
-#include "pixmaps/stock_mail_reply.xpm"
-#include "pixmaps/stock_mail_reply_to_all.xpm"
-#include "pixmaps/stock_mail_forward.xpm"
-#include "pixmaps/stock_mail_send.xpm"
-#include "pixmaps/stock_preferences.xpm"
-#include "pixmaps/stock_properties.xpm"
-#include "pixmaps/stock_down_arrow.xpm"
-#include "pixmaps/stock_close.xpm"
-#include "pixmaps/stock_exec.xpm"
-
-#define CREATE_TOOLBAR_ICON(xpm_d) \
-{ \
-	icon = gdk_pixmap_create_from_xpm_d(container->window, &mask, \
-					    &container->style->white, \
-					    xpm_d); \
-	icon_wid = gtk_pixmap_new(icon, mask); \
-}
-
 static void main_window_toolbar_create(MainWindow *mainwin,
 				       GtkWidget *container)
 {
 	GtkWidget *toolbar;
-	GdkPixmap *icon;
-	GdkBitmap *mask;
 	GtkWidget *icon_wid;
 	GtkWidget *get_btn;
 	GtkWidget *getall_btn;
@@ -1514,13 +1508,13 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 	gtk_toolbar_set_space_style(GTK_TOOLBAR(toolbar),
 				    GTK_TOOLBAR_SPACE_LINE);
 
-	CREATE_TOOLBAR_ICON(stock_mail_receive_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_RECEIVE);
 	get_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					  _("Get"),
 					  _("Incorporate new mail"),
 					  "Get",
 					  icon_wid, toolbar_inc_cb, mainwin);
-	CREATE_TOOLBAR_ICON(stock_mail_receive_all_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_RECEIVE_ALL);
 	getall_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					     _("Get all"),
 					     _("Incorporate new mail of all accounts"),
@@ -1531,7 +1525,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
-	CREATE_TOOLBAR_ICON(stock_mail_send_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_SEND);
 	send_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					   _("Send"),
 					   _("Send queued message(s)"),
@@ -1542,7 +1536,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
-	CREATE_TOOLBAR_ICON(stock_mail_compose_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_COMPOSE);
 	compose_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					      _("Compose"),
 					      _("Compose new message"),
@@ -1550,7 +1544,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					      icon_wid,
 					      toolbar_compose_cb,
 					      mainwin);
-	CREATE_TOOLBAR_ICON(stock_mail_reply_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_REPLY);
 	reply_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					    _("Reply"),
 					    _("Reply to the message"),
@@ -1558,7 +1552,8 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					    icon_wid,
 					    toolbar_reply_cb,
 					    mainwin);
-	CREATE_TOOLBAR_ICON(stock_mail_reply_to_all_xpm);
+	icon_wid = stock_pixmap_widget
+		(container, STOCK_PIXMAP_MAIL_REPLY_TO_ALL);
 	replyall_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					       _("Reply all"),
 					       _("Reply to all"),
@@ -1566,7 +1561,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					       icon_wid,
 					       toolbar_reply_to_all_cb,
 					       mainwin);
-	CREATE_TOOLBAR_ICON(stock_mail_forward_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_MAIL_FORWARD);
 	fwd_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					  _("Forward"),
 					  _("Forward the message"),
@@ -1577,7 +1572,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
-	CREATE_TOOLBAR_ICON(stock_close_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_CLOSE);
 	delete_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					  _("Delete"),
 					  _("Delete the message"),
@@ -1586,7 +1581,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					  toolbar_delete_cb,
 					  mainwin);
 
-	CREATE_TOOLBAR_ICON(stock_exec_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_EXEC);
 	exec_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					   _("Execute"),
 					   _("Execute marked process"),
@@ -1595,7 +1590,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					   toolbar_exec_cb,
 					   mainwin);
 
-	CREATE_TOOLBAR_ICON(stock_down_arrow_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_DOWN_ARROW);
 	next_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					   _("Next"),
 					   _("Next unread message"),
@@ -1606,7 +1601,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 
 	gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
 
-	CREATE_TOOLBAR_ICON(stock_preferences_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_PREFERENCES);
 	prefs_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					    _("Prefs"),
 					    _("Common preference"),
@@ -1614,7 +1609,7 @@ static void main_window_toolbar_create(MainWindow *mainwin,
 					    icon_wid,
 					    toolbar_prefs_cb,
 					    mainwin);
-	CREATE_TOOLBAR_ICON(stock_properties_xpm);
+	icon_wid = stock_pixmap_widget(container, STOCK_PIXMAP_PROPERTIES);
 	account_btn = gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
 					      _("Account"),
 					      _("Account setting"),
@@ -2157,13 +2152,13 @@ static void thread_cb(MainWindow *mainwin, guint action, GtkWidget *widget)
 	if (0 == action) {
 		summary_thread_build(mainwin->summaryview);
 		mainwin->summaryview->folder_item->threaded = TRUE;
-		menu_set_sensitive(ifactory, "/Summary/Thread view",   FALSE);
-		menu_set_sensitive(ifactory, "/Summary/Unthread view", TRUE);
+		menu_set_sensitive(ifactory, "/View/Thread view",   FALSE);
+		menu_set_sensitive(ifactory, "/View/Unthread view", TRUE);
 	} else {
 		summary_unthread(mainwin->summaryview);
 		mainwin->summaryview->folder_item->threaded = FALSE;
-		menu_set_sensitive(ifactory, "/Summary/Thread view",   TRUE);
-		menu_set_sensitive(ifactory, "/Summary/Unthread view", FALSE);
+		menu_set_sensitive(ifactory, "/View/Thread view",   TRUE);
+		menu_set_sensitive(ifactory, "/View/Unthread view", FALSE);
 	}
 }
 
