@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2001 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -110,7 +110,8 @@ MessageView *messageview_create_with_new_window(void)
 	MessageView *msgview;
 
 	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title(GTK_WINDOW(window), "Message");
+	gtk_window_set_title(GTK_WINDOW(window), _("Sylpheed - Message View"));
+	gtk_window_set_wmclass(GTK_WINDOW(window), "message_view", "Sylpheed");
 	gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, FALSE);
 	gtk_widget_set_usize(window, prefs_common.msgwin_width,
 			     prefs_common.msgwin_height);
@@ -217,7 +218,7 @@ static void messageview_change_view_type(MessageView *messageview,
 	if (messageview->type == type) return;
 
 	if (type == MVIEW_MIME) {
-		gtk_container_remove
+		gtkut_container_remove
 			(GTK_CONTAINER(GTK_WIDGET_PTR(messageview)),
 			 GTK_WIDGET_PTR(textview));
 		gtk_box_pack_start(GTK_BOX(messageview->vbox),
@@ -226,16 +227,16 @@ static void messageview_change_view_type(MessageView *messageview,
 				  GTK_WIDGET_PTR(textview));
 		mimeview->type = MIMEVIEW_TEXT;
 	} else if (type == MVIEW_TEXT) {
-		gtk_container_remove
+		gtkut_container_remove
 			(GTK_CONTAINER(GTK_WIDGET_PTR(messageview)),
 			 GTK_WIDGET_PTR(mimeview));
 
 		if (mimeview->vbox == GTK_WIDGET_PTR(textview)->parent) {
-			gtk_container_remove(GTK_CONTAINER(mimeview->vbox),
-					     GTK_WIDGET_PTR(textview));
+			gtkut_container_remove(GTK_CONTAINER(mimeview->vbox),
+			 		       GTK_WIDGET_PTR(textview));
 		} else {
-			gtk_container_remove(GTK_CONTAINER(mimeview->vbox),
-					     GTK_WIDGET_PTR(imageview));
+			gtkut_container_remove(GTK_CONTAINER(mimeview->vbox),
+			  		       GTK_WIDGET_PTR(imageview));
 		}
 
 		gtk_box_pack_start(GTK_BOX(messageview->vbox),
