@@ -29,6 +29,7 @@
 #include <stdlib.h>
 #include <dirent.h>
 #include <unistd.h>
+#include <time.h>
 
 #include "intl.h"
 #include "news.h"
@@ -100,13 +101,14 @@ static Session *news_session_new(const gchar *server, gushort port,
 		return NULL;
 
 	session = g_new(NNTPSession, 1);
-	SESSION(session)->type      = SESSION_NEWS;
-	SESSION(session)->server    = g_strdup(server);
-	session->nntp_sock          = nntp_sock;
-	SESSION(session)->sock      = nntp_sock->sock;
-	SESSION(session)->connected = TRUE;
-	SESSION(session)->phase     = SESSION_READY;
-	SESSION(session)->data      = NULL;
+	SESSION(session)->type             = SESSION_NEWS;
+	SESSION(session)->server           = g_strdup(server);
+	session->nntp_sock                 = nntp_sock;
+	SESSION(session)->sock             = nntp_sock->sock;
+	SESSION(session)->connected        = TRUE;
+	SESSION(session)->phase            = SESSION_READY;
+	SESSION(session)->last_access_time = time(NULL);
+	SESSION(session)->data             = NULL;
 	session->group = NULL;
 
 	return SESSION(session);
@@ -163,34 +165,39 @@ static Session *news_session_new_for_folder(Folder *folder)
 
 NNTPSession *news_session_get(Folder *folder)
 {
-	NNTPSession *session;
+	RemoteFolder *rfolder = REMOTE_FOLDER(folder);
 
 	g_return_val_if_fail(folder != NULL, NULL);
 	g_return_val_if_fail(folder->type == F_NEWS, NULL);
 	g_return_val_if_fail(folder->account != NULL, NULL);
 
-	if (!REMOTE_FOLDER(folder)->session) {
-		REMOTE_FOLDER(folder)->session =
-			news_session_new_for_folder(folder);
+	if (!rfolder->session) {
+		rfolder->session = news_session_new_for_folder(folder);
 		statusbar_pop_all();
-		return NNTP_SESSION(REMOTE_FOLDER(folder)->session);
+		return NNTP_SESSION(rfolder->session);
 	}
 
-	session = NNTP_SESSION(REMOTE_FOLDER(folder)->session);
+	if (time(NULL) - rfolder->session->last_access_time < SESSION_TIMEOUT) {
+		rfolder->session->last_access_time = time(NULL);
+		statusbar_pop_all();
+		return NNTP_SESSION(rfolder->session);
+	}
 
-	if (nntp_mode(session->nntp_sock, FALSE) != NN_SUCCESS) {
+	if (nntp_mode(NNTP_SESSION(rfolder->session)->nntp_sock, FALSE)
+	    != NN_SUCCESS) {
 		log_warning(_("NNTP connection to %s:%d has been"
 			      " disconnected. Reconnecting...\n"),
 			    folder->account->nntp_server,
 			    folder->account->set_nntpport ?
 			    folder->account->nntpport : NNTP_PORT);
-		session_destroy(REMOTE_FOLDER(folder)->session);
-		REMOTE_FOLDER(folder)->session =
-			news_session_new_for_folder(folder);
+		session_destroy(rfolder->session);
+		rfolder->session = news_session_new_for_folder(folder);
 	}
 
+	if (rfolder->session)
+		rfolder->session->last_access_time = time(NULL);
 	statusbar_pop_all();
-	return NNTP_SESSION(REMOTE_FOLDER(folder)->session);
+	return NNTP_SESSION(rfolder->session);
 }
 
 GSList *news_get_article_list(Folder *folder, FolderItem *item,
@@ -289,6 +296,32 @@ void news_scan_group(Folder *folder, FolderItem *item)
 {
 }
 
+static NewsGroupInfo *news_group_info_new(const gchar *name,
+					  gint first, gint last, gchar type)
+{
+	NewsGroupInfo *ginfo;
+
+	ginfo = g_new(NewsGroupInfo, 1);
+	ginfo->name = g_strdup(name);
+	ginfo->first = first;
+	ginfo->last = last;
+	ginfo->type = type;
+
+	return ginfo;
+}
+
+static void news_group_info_free(NewsGroupInfo *ginfo)
+{
+	g_free(ginfo->name);
+	g_free(ginfo);
+}
+
+static gint news_group_info_compare(NewsGroupInfo *ginfo1,
+				    NewsGroupInfo *ginfo2)
+{
+	return g_strcasecmp(ginfo1->name, ginfo2->name);
+}
+
 GSList *news_get_group_list(Folder *folder)
 {
 	gchar *path, *filename;
@@ -336,12 +369,27 @@ GSList *news_get_group_list(Folder *folder)
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		gchar *p = buf;
-		while (*p != '\0' && *p != ' ') p++;
+		gchar *name;
+		gint last_num;
+		gint first_num;
+		gchar type;
+		NewsGroupInfo *ginfo;
+
+		p = strchr(p, ' ');
+		if (!p) continue;
 		*p = '\0';
+		p++;
+		name = buf;
+
+		if (sscanf(p, "%d %d %c", &last_num, &first_num, &type) < 3)
+			continue;
+
+		ginfo = news_group_info_new(name, first_num, last_num, type);
+
 		if (!last)
-			last = list = g_slist_append(NULL, g_strdup(buf));
+			last = list = g_slist_append(NULL, ginfo);
 		else {
-			last = g_slist_append(last, g_strdup(buf));
+			last = g_slist_append(last, ginfo);
 			last = last->next;
 		}
 	}
@@ -349,14 +397,25 @@ GSList *news_get_group_list(Folder *folder)
 	fclose(fp);
 	g_free(filename);
 
-	list = g_slist_sort(list, (GCompareFunc)g_strcasecmp);
+	list = g_slist_sort(list, (GCompareFunc)news_group_info_compare);
 
 	statusbar_pop_all();
 
 	return list;
 }
 
-void news_remove_group_list(Folder *folder)
+void news_group_list_free(GSList *group_list)
+{
+	GSList *cur;
+
+	if (!group_list) return;
+
+	for (cur = group_list; cur != NULL; cur = cur->next)
+		news_group_info_free((NewsGroupInfo *)cur->data);
+	g_slist_free(group_list);
+}
+
+void news_remove_group_list_cache(Folder *folder)
 {
 	gchar *path, *filename;
 
