@@ -27,6 +27,7 @@
 #include "nntp.h"
 
 typedef struct _NNTPSession	NNTPSession;
+typedef struct _NewsGroupInfo	NewsGroupInfo;
 
 #define NNTP_SESSION(obj)	((NNTPSession *)obj)
 
@@ -38,6 +39,13 @@ struct _NNTPSession
 	gchar *group;
 };
 
+struct _NewsGroupInfo
+{
+	gchar *name;
+	gchar first;
+	gchar last;
+	gchar type;
+};
 
 void news_session_destroy		(NNTPSession	*session);
 NNTPSession *news_session_get		(Folder		*folder);
@@ -53,7 +61,8 @@ void news_scan_group			(Folder		*folder,
 					 FolderItem	*item);
 
 GSList *news_get_group_list		(Folder		*folder);
-void news_remove_group_list		(Folder		*folder);
+void news_group_list_free		(GSList		*group_list);
+void news_remove_group_list_cache	(Folder		*folder);
 
 gint news_post				(Folder		*folder,
 					 const gchar	*file);
