@@ -35,6 +35,7 @@
 #include "base64.h"
 #include "uuencode.h"
 #include "unmime.h"
+#include "html.h"
 #include "codeconv.h"
 #include "utils.h"
 #include "prefs_common.h"
@@ -44,7 +45,7 @@
 #endif
 
 static GHashTable *procmime_get_mime_type_table	(void);
-static GList *procmime_get_mime_type_list	(void);
+static GList *procmime_get_mime_type_list	(const gchar *file);
 
 
 MimeInfo *procmime_mimeinfo_new(void)
@@ -134,14 +135,8 @@ void procmime_mimeinfo_replace(MimeInfo *old, MimeInfo *new)
 	MimeInfo *parent = old->parent;
 	MimeInfo *child;
 
-	if (!parent) {
-		g_warning("oops: Not top message");
-		return;
-	}
-	if (new->next) {
-		g_message("oops: new child should not have a sibling");
-		return;
-	}
+	g_return_if_fail(parent != NULL);
+	g_return_if_fail(new->next == NULL);
 
 	for (child = parent->children; child && child != old;
 	     child = child->next)
@@ -161,6 +156,31 @@ void procmime_mimeinfo_replace(MimeInfo *old, MimeInfo *new)
 	}
 }
 
+MimeInfo *procmime_mimeinfo_next(MimeInfo *mimeinfo)
+{
+	if (!mimeinfo) return NULL;
+
+	if (mimeinfo->children)
+		return mimeinfo->children;
+	if (mimeinfo->sub)
+		return mimeinfo->sub;
+	if (mimeinfo->next)
+		return mimeinfo->next;
+
+	for (mimeinfo = mimeinfo->parent; mimeinfo != NULL;
+	     mimeinfo = mimeinfo->parent) {
+		if (mimeinfo->next)
+			return mimeinfo->next;
+		if (mimeinfo->main) {
+			mimeinfo = mimeinfo->main;
+			if (mimeinfo->next)
+				return mimeinfo->next;
+		}
+	}
+
+	return NULL;
+}
+
 MimeInfo *procmime_scan_message(MsgInfo *msginfo)
 {
 	FILE *fp;
@@ -270,7 +290,6 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 		if (eom) break;
 	}
-	//g_message ("** at " __PRETTY_FUNCTION__ ":%d:", __LINE__);
 }
 
 void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
@@ -613,13 +632,79 @@ gint procmime_get_part(const gchar *outfile, const gchar *infile,
 	return 0;
 }
 
-FILE *procmime_get_text_part(MsgInfo *msginfo)
+FILE *procmime_get_text_content(MimeInfo *mimeinfo, FILE *infp)
 {
-	FILE *infp, *tmpfp, *outfp;
-	MimeInfo *mimeinfo, *partinfo = NULL;
+	FILE *tmpfp, *outfp;
 	gchar *src_codeset;
 	gboolean conv_fail = FALSE;
 	gchar buf[BUFFSIZE];
+	gchar *str;
+
+	g_return_val_if_fail(mimeinfo != NULL, NULL);
+	g_return_val_if_fail(infp != NULL, NULL);
+	g_return_val_if_fail(mimeinfo->mime_type == MIME_TEXT ||
+			     mimeinfo->mime_type == MIME_TEXT_HTML, NULL);
+
+	if (fseek(infp, mimeinfo->fpos, SEEK_SET) < 0) {
+		perror("fseek");
+		return NULL;
+	}
+
+	while (fgets(buf, sizeof(buf), infp) != NULL)
+		if (buf[0] == '\r' || buf[0] == '\n') break;
+
+	tmpfp = procmime_decode_content(NULL, infp, mimeinfo);
+	if (!tmpfp)
+		return NULL;
+
+	if ((outfp = my_tmpfile()) == NULL) {
+		perror("tmpfile");
+		fclose(tmpfp);
+		return NULL;
+	}
+
+	src_codeset = prefs_common.force_charset
+		? prefs_common.force_charset : mimeinfo->charset;
+
+	if (mimeinfo->mime_type == MIME_TEXT) {
+		while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
+			str = conv_codeset_strdup(buf, src_codeset, NULL);
+			if (str) {
+				fputs(str, outfp);
+				g_free(str);
+			} else {
+				conv_fail = TRUE;
+				fputs(buf, outfp);
+			}
+		}
+	} else if (mimeinfo->mime_type == MIME_TEXT_HTML) {
+		HTMLParser *parser;
+		CodeConverter *conv;
+
+		conv = conv_code_converter_new(src_codeset);
+		parser = html_parser_new(tmpfp, conv);
+		while ((str = html_parse(parser)) != NULL) {
+			fputs(str, outfp);
+		}
+		html_parser_destroy(parser);
+		conv_code_converter_destroy(conv);
+	}
+
+	if (conv_fail)
+		g_warning(_("procmime_get_text_content(): Code conversion failed.\n"));
+
+	fclose(tmpfp);
+	rewind(outfp);
+
+	return outfp;
+}
+
+/* search the first text part of (multipart) MIME message,
+   decode, convert it and output to outfp. */
+FILE *procmime_get_first_text_content(MsgInfo *msginfo)
+{
+	FILE *infp, *outfp = NULL;
+	MimeInfo *mimeinfo, *partinfo;
 
 	g_return_val_if_fail(msginfo != NULL, NULL);
 
@@ -631,68 +716,98 @@ FILE *procmime_get_text_part(MsgInfo *msginfo)
 		return NULL;
 	}
 
-	if (mimeinfo->mime_type == MIME_MULTIPART) {
-		partinfo = mimeinfo->children;
-		if (partinfo && partinfo->mime_type == MIME_TEXT) {
-			if (fseek(infp, partinfo->fpos, SEEK_SET) < 0) {
-				perror("fseek");
-				partinfo = NULL;
-			}
-		} else
-			partinfo = NULL;
-	} else if (mimeinfo->mime_type == MIME_TEXT) {
-		partinfo = mimeinfo;
-		if (fseek(infp, partinfo->fpos, SEEK_SET) < 0) {
-			perror("fseek");
-			partinfo = NULL;
-		}
-	}
-
+	partinfo = mimeinfo;
+	while (partinfo && partinfo->mime_type != MIME_TEXT)
+		partinfo = procmime_mimeinfo_next(partinfo);
 	if (!partinfo) {
-		procmime_mimeinfo_free_all(mimeinfo);
-		return NULL;
+		partinfo = mimeinfo;
+		while (partinfo && partinfo->mime_type != MIME_TEXT_HTML)
+			partinfo = procmime_mimeinfo_next(partinfo);
 	}
 
-	while (fgets(buf, sizeof(buf), infp) != NULL)
-		if (buf[0] == '\r' || buf[0] == '\n') break;
+	if (partinfo)
+		outfp = procmime_get_text_content(partinfo, infp);
 
-	tmpfp = procmime_decode_content(NULL, infp, partinfo);
-	if (!tmpfp) {
-		procmime_mimeinfo_free_all(mimeinfo);
-		return NULL;
-	}
-
-	if ((outfp = my_tmpfile()) == NULL) {
-		perror("tmpfile");
-		fclose(tmpfp);
-		procmime_mimeinfo_free_all(mimeinfo);
-		return NULL;
-	}
-
-	src_codeset = prefs_common.force_charset
-		? prefs_common.force_charset : partinfo->charset;
-
-	while (fgets(buf, sizeof(buf), tmpfp) != NULL) {
-		gchar *str;
-
-		str = conv_codeset_strdup(buf, src_codeset, NULL);
-		if (str) {
-			fputs(str, outfp);
-			g_free(str);
-		} else {
-			conv_fail = TRUE;
-			fputs(buf, outfp);
-		}
-	}
-	if (conv_fail) g_warning(_("Code conversion failed.\n"));
-
-	fclose(tmpfp);
+	fclose(infp);
 	procmime_mimeinfo_free_all(mimeinfo);
-	rewind(outfp);
 
 	return outfp;
 }
 
+gboolean procmime_find_string_part(MimeInfo *mimeinfo, const gchar *filename,
+				   const gchar *str, gboolean case_sens)
+{
+
+	FILE *infp, *outfp;
+	gchar buf[BUFFSIZE];
+	gchar *(* StrFindFunc) (const gchar *haystack, const gchar *needle);
+
+	g_return_val_if_fail(mimeinfo != NULL, FALSE);
+	g_return_val_if_fail(mimeinfo->mime_type == MIME_TEXT ||
+			     mimeinfo->mime_type == MIME_TEXT_HTML, FALSE);
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	if ((infp = fopen(filename, "r")) == NULL) {
+		FILE_OP_ERROR(filename, "fopen");
+		return FALSE;
+	}
+
+	outfp = procmime_get_text_content(mimeinfo, infp);
+	fclose(infp);
+
+	if (!outfp)
+		return FALSE;
+
+	if (case_sens)
+		StrFindFunc = strstr;
+	else
+		StrFindFunc = strcasestr;
+
+	while (fgets(buf, sizeof(buf), outfp) != NULL) {
+		if (StrFindFunc(buf, str) != NULL) {
+			fclose(outfp);
+			return TRUE;
+		}
+	}
+
+	fclose(outfp);
+
+	return FALSE;
+}
+
+gboolean procmime_find_string(MsgInfo *msginfo, const gchar *str,
+			      gboolean case_sens)
+{
+	MimeInfo *mimeinfo;
+	MimeInfo *partinfo;
+	gchar *filename;
+	gboolean found = FALSE;
+
+	g_return_val_if_fail(msginfo != NULL, FALSE);
+	g_return_val_if_fail(str != NULL, FALSE);
+
+	filename = procmsg_get_message_file(msginfo);
+	if (!filename) return FALSE;
+	mimeinfo = procmime_scan_message(msginfo);
+
+	for (partinfo = mimeinfo; partinfo != NULL;
+	     partinfo = procmime_mimeinfo_next(partinfo)) {
+		if (partinfo->mime_type == MIME_TEXT ||
+		    partinfo->mime_type == MIME_TEXT_HTML) {
+			if (procmime_find_string_part
+				(partinfo, filename, str, case_sens) == TRUE) {
+				found = TRUE;
+				break;
+			}
+		}
+	}
+
+	procmime_mimeinfo_free_all(mimeinfo);
+	g_free(filename);
+
+	return found;
+}
+
 gchar *procmime_get_tmp_file_name(MimeInfo *mimeinfo)
 {
 	static guint32 id = 0;
@@ -789,30 +904,53 @@ static GHashTable *procmime_get_mime_type_table(void)
 	gchar **exts;
 
 	if (!mime_type_list) {
-		mime_type_list = procmime_get_mime_type_list();
-		if (!mime_type_list) return NULL;
+		GList *list;
+		gchar *dir;
+
+		mime_type_list =
+			procmime_get_mime_type_list(SYSCONFDIR "/mime.types");
+		if (!mime_type_list) {
+			list = procmime_get_mime_type_list("/etc/mime.types");
+			mime_type_list = g_list_concat(mime_type_list, list);
+		}
+		dir = g_strconcat(get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
+				  G_DIR_SEPARATOR_S, "mime.types", NULL);
+		list = procmime_get_mime_type_list(dir);
+		g_free(dir);
+		mime_type_list = g_list_concat(mime_type_list, list);
+
+		if (!mime_type_list) {
+			g_warning("mime.types not found\n");
+			return NULL;
+		}
 	}
 
 	table = g_hash_table_new(g_str_hash, g_str_equal);
 
 	for (cur = mime_type_list; cur != NULL; cur = cur->next) {
 		gint i;
+		gchar *key;
 
 		mime_type = (MimeType *)cur->data;
 
 		if (!mime_type->extension) continue;
 
 		exts = g_strsplit(mime_type->extension, " ", 16);
-		for (i = 0; exts[i] != NULL; i++)
-			g_hash_table_insert(table, g_strdup(exts[i]),
-					    mime_type);
+		for (i = 0; exts[i] != NULL; i++) {
+			/* use previously dup'd key on overwriting */
+			if (g_hash_table_lookup(table, exts[i]))
+				key = exts[i];
+			else
+				key = g_strdup(exts[i]);
+			g_hash_table_insert(table, key, mime_type);
+		}
 		g_strfreev(exts);
 	}
 
 	return table;
 }
 
-static GList *procmime_get_mime_type_list(void)
+static GList *procmime_get_mime_type_list(const gchar *file)
 {
 	GList *list = NULL;
 	FILE *fp;
@@ -820,10 +958,9 @@ static GList *procmime_get_mime_type_list(void)
 	gchar *p, *delim;
 	MimeType *mime_type;
 
-	if ((fp = fopen("/etc/mime.types", "r")) == NULL) {
-		FILE_OP_ERROR("/etc/mime.types", "fopen");
-		return NULL;
-	}
+	if ((fp = fopen(file, "r")) == NULL) return NULL;
+
+	debug_print("Reading %s ...\n", file);
 
 	while (fgets(buf, sizeof(buf), fp) != NULL) {
 		p = strchr(buf, '#');
