@@ -149,7 +149,7 @@ void quote_fmt_init(MsgInfo *info, gchar *my_quote_str)
 
 void quote_fmterror(char *str)
 {
-	g_warning(_("Error %s\n"), str);
+	g_warning(_("Error: %s\n"), str);
 	error = 1;
 }
 
@@ -1118,13 +1118,13 @@ case 27:
 case 28:
 #line 359 "quote_fmt_parse.y"
 {
-		INSERT("(");
+		INSERT("{");
 	;
     break;}
 case 29:
 #line 363 "quote_fmt_parse.y"
 {
-		INSERT(")");
+		INSERT("}");
 	;
     break;}
 case 30:
