@@ -27,7 +27,8 @@
 #include <gtk/gtkctree.h>
 #include <gtk/gtkdnd.h>
 
-typedef struct _SummaryView	SummaryView;
+typedef struct _SummaryView		SummaryView;
+typedef struct _SummaryColumnState	SummaryColumnState;
 
 #include "mainwindow.h"
 #include "folderview.h"
@@ -39,15 +40,15 @@ typedef struct _SummaryView	SummaryView;
 
 typedef enum
 {
-	S_COL_MARK	= 0,
-	S_COL_UNREAD	= 1,
-	S_COL_MIME	= 2,
-	S_COL_NUMBER	= 3,
-	S_COL_SIZE	= 4,
-	S_COL_DATE	= 5,
-	S_COL_FROM	= 6,
-	S_COL_SUBJECT	= 7
-} SummaryColumnPos;
+	S_COL_MARK,
+	S_COL_UNREAD,
+	S_COL_MIME,
+	S_COL_SUBJECT,
+	S_COL_FROM,
+	S_COL_DATE,
+	S_COL_SIZE,
+	S_COL_NUMBER,
+} SummaryColumnType;
 
 #define N_SUMMARY_COLS	8
 
@@ -82,6 +83,12 @@ typedef enum
 
 extern GtkTargetEntry summary_drag_types[1];
 
+struct _SummaryColumnState
+{
+	SummaryColumnType type;
+	gboolean visible;
+};
+
 struct _SummaryView
 {
 	GtkWidget *vbox;
@@ -106,9 +113,14 @@ struct _SummaryView
 	gboolean msg_is_toggled_on;
 	gboolean display_msg;
 
+	SummaryColumnState col_state[N_SUMMARY_COLS];
+	gint col_pos[N_SUMMARY_COLS];
+
 	GdkColor color_marked;
 	GdkColor color_dim;
 
+	guint lock_count;
+
 	MainWindow   *mainwin;
 	FolderView   *folderview;
 	HeaderView   *headerview;
@@ -151,6 +163,10 @@ gboolean summary_show		  (SummaryView		*summaryview,
 void summary_clear_list		  (SummaryView		*summaryview);
 void summary_clear_all		  (SummaryView		*summaryview);
 
+void summary_lock		  (SummaryView		*summaryview);
+void summary_unlock		  (SummaryView		*summaryview);
+gboolean summary_is_locked	  (SummaryView		*summaryview);
+
 SummarySelection summary_get_selection_type	(SummaryView	*summaryview);
 
 void summary_select_prev_unread	  (SummaryView		*summaryview);
@@ -161,21 +177,29 @@ void summary_select_prev_labeled  (SummaryView		*summaryview);
 void summary_select_next_labeled  (SummaryView		*summaryview);
 void summary_select_by_msgnum	  (SummaryView		*summaryview,
 				   guint		 msgnum);
-void summary_thread_build	  (SummaryView		*summaryview,
-				   gboolean		 init);
+void summary_select_node	  (SummaryView		*summaryview,
+				   GtkCTreeNode		*node,
+				   gboolean		 display_msg);
+
+void summary_thread_build	  (SummaryView		*summaryview);
 void summary_unthread		  (SummaryView		*summaryview);
+
 void summary_filter		  (SummaryView		*summaryview);
 void summary_sort		  (SummaryView		*summaryview,
 				   SummarySortType	 type);
 
 void summary_delete		  (SummaryView		*summaryview);
 void summary_delete_duplicated	  (SummaryView		*summaryview);
-void summary_execute		  (SummaryView		*summaryview);
+
+gboolean summary_execute	  (SummaryView		*summaryview);
+
 void summary_attract_by_subject	  (SummaryView		*summaryview);
+
 gint summary_write_cache	  (SummaryView		*summaryview);
+
 void summary_pass_key_press_event (SummaryView		*summaryview,
 				   GdkEventKey		*event);
-void summary_change_display_item  (SummaryView		*summaryview);
+
 void summary_redisplay_msg	  (SummaryView		*summaryview);
 void summary_open_msg		  (SummaryView		*summaryview);
 void summary_view_source	  (SummaryView		*summaryview);
@@ -208,4 +232,6 @@ void summary_set_colorlabel_color (GtkCTree		*ctree,
 				   GtkCTreeNode		*node,
 				   guint		 labelcolor);
 
+void summary_set_column_order	  (SummaryView		*summaryview);
+
 #endif /* __SUMMARY_H__ */
