@@ -111,6 +111,24 @@
 	ptr = __tmp; \
 }
 
+#define Xstrcat_a(ptr, str1, str2, iffail) \
+{ \
+	gchar *__tmp; \
+	gint len1, len2; \
+ \
+	len1 = strlen(str1); \
+	len2 = strlen(str2); \
+	if ((__tmp = alloca(len1 + len2 + 1)) == NULL) { \
+		g_warning("can't allocate memory\n"); \
+		iffail; \
+	} else { \
+		memcpy(__tmp, str1, len1); \
+		memcpy(__tmp + len1, str2, len2 + 1); \
+	} \
+ \
+	ptr = __tmp; \
+}
+
 #define FILE_OP_ERROR(file, func) \
 { \
 	fprintf(stderr, "%s: ", file); \
@@ -242,13 +260,16 @@ void subst_char				(gchar		*str,
 gboolean is_header_line			(const gchar	*str);
 gboolean is_ascii_str			(const guchar	*str);
 gint get_quote_level			(const gchar	*str);
-GList *uri_list_extract_filenames	(const gchar	*uri_list);
 gchar *strstr_with_skip_quote		(const gchar	*haystack,
 					 const gchar	*needle);
 gchar **strsplit_with_quote		(const gchar	*str,
 					 const gchar	*delim,
 					 gint		 max_tokens);
 
+GList *uri_list_extract_filenames	(const gchar	*uri_list);
+void decode_uri				(gchar		*decoded_uri,
+					 const gchar	*encoded_uri);
+
 /* return static strings */
 gchar *get_home_dir		(void);
 gchar *get_rc_dir		(void);
